package cz.moneta.test.regression.backoffice.ufooperations.regressionsbbo;

import cz.moneta.test.dsl.Harness;
import cz.moneta.test.dsl.ufo.operations.search.SearchBy;
import cz.moneta.test.dsl.util.DateUtils;
import cz.moneta.test.dsl.util.credentials.Keys;
import cz.moneta.test.dsl.util.task.ufo.operations.OperationsTasks;
import cz.moneta.test.harness.annotations.JiraTestCase;
import cz.moneta.test.harness.annotations.ParameterizedTestCase;
import cz.moneta.test.harness.annotations.TestScenarioWithOrder;
import cz.moneta.test.harness.support.auth.Credentials;
import cz.moneta.test.harness.support.auth.Key;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.params.provider.ValueSource;

import static cz.moneta.test.dsl.util.task.ufo.cb.CommercialTasks.JIRA_COMMENT_STORE_KEY;
import static cz.moneta.test.harness.annotations.Environment.PPE;

@TestScenarioWithOrder(name = "Print Templates Gdpr")
public class PrintTemplatesGdpr {

    String department = "OPERATIONS_BACKOFFICE_ZAKLAD";
    String category = "GDPR";
    String CIF;

    @Order(1)
    @ParameterizedTestCase(name = "Print template GDPR another address", environments = {PPE})
    @ValueSource(strings = {
            "Právo na výmaz - 1 rok",
            "Právo na výmaz - 5 let",
            "Právo na výmaz - 10 let",
            "Přístup k údajům - zamítnuto",
            "Přístup k údajům - uznáno",
            "Právo na výmaz - ukončený vztah",
            "Právo na výmaz - již vymazáno",
            "Právo na výmaz - existující vztah",
            "Právo na výmaz - bude vymazáno"
    })
    @JiraTestCase(id = "BOC-T8763")
    public void printTemplatesGdprAnotherAddress(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.CIF = "3000006024";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .fillSearchInput(CIF)
                .searchBy(SearchBy.CIF)
                .onClientDetailsPage()
                .clickPrintLettersWithCif()
                .selectDepartment(department)
                .selectCategory(category)
                .selectTemplate(printName)
                .selectGenderIfVisible()
                .clickAnotherAddress()
                .fillNameInput("David Novák")
                .fillStreetInput("Dlouhá")
                .fillBuildingNumberInput("928")
                .fillStreetNumberInput("1")
                .fillCityInput("Praha")
                .fillPostalCodeInput("11000")
                .clickPrintWithAnotherAddress()
                .clickAddress()
                .clickPrint()
                .clickSaveToDmsIfNotChecked()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(2)
    @ParameterizedTestCase(name = "Print template GDPR DMS", environments = {PPE})
    @ValueSource(strings = {
            "Právo na výmaz - 1 rok",
            "Právo na výmaz - 5 let",
            "Právo na výmaz - 10 let",
            "Přístup k údajům - zamítnuto",
            "Přístup k údajům - uznáno",
            "Právo na výmaz - ukončený vztah",
            "Právo na výmaz - již vymazáno",
            "Právo na výmaz - existující vztah",
            "Právo na výmaz - bude vymazáno"
    })
    @JiraTestCase(id = "BOC-T8761")
    public void printTemplatesGdprDms(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.CIF = "3000060550";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithCif(CIF, department, category, printName))
                .clickSaveToDmsIfNotChecked()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(3)
    @ParameterizedTestCase(name = "Print template GDPR Dopis", environments = {PPE})
    @ValueSource(strings = {
            "Právo na výmaz - 1 rok",
            "Právo na výmaz - 5 let",
            "Právo na výmaz - 10 let",
            "Přístup k údajům - zamítnuto",
            "Přístup k údajům - uznáno",
            "Právo na výmaz - ukončený vztah",
            "Právo na výmaz - již vymazáno",
            "Právo na výmaz - existující vztah",
            "Právo na výmaz - bude vymazáno"
    })
    @JiraTestCase(id = "BOC-T8765")
    public void printTemplatesGdprLetter(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.CIF = "9002004065";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithCif(CIF, department, category, printName))
                .clickSaveToEts()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(4)
    @ParameterizedTestCase(name = "Print template GDPR Ib", environments = {PPE})
    @ValueSource(strings = {
            "Právo na výmaz - 1 rok",
            "Právo na výmaz - 5 let",
            "Právo na výmaz - 10 let",
            "Přístup k údajům - zamítnuto",
            "Přístup k údajům - uznáno",
            "Právo na výmaz - ukončený vztah",
            "Právo na výmaz - již vymazáno",
            "Právo na výmaz - existující vztah",
            "Právo na výmaz - bude vymazáno"
    })
    @JiraTestCase(id = "BOC-T8762")
    public void printTemplatesGdprIb(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.CIF = "3000233808";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithCif(CIF, department, category, printName))
                .clickSaveToIbIfNotChecked()
                .clickSend()
                .onClientDetailPage()
                .clickCorrespondence()
                .assertPrintName(printName)
                .assertDate(DateUtils.getFormattedTodayDate("d.M.yyyy"));
    }

    @Order(5)
    @ParameterizedTestCase(name = "Print template GDPR Local", environments = {PPE})
    @ValueSource(strings = {
            "Právo na výmaz - 1 rok",
            "Právo na výmaz - 5 let",
            "Právo na výmaz - 10 let",
            "Přístup k údajům - zamítnuto",
            "Přístup k údajům - uznáno",
            "Právo na výmaz - ukončený vztah",
            "Právo na výmaz - již vymazáno",
            "Právo na výmaz - existující vztah",
            "Právo na výmaz - bude vymazáno"
    })
    @JiraTestCase(id = "BOC-T8764")
    public void printTemplatesGdprLocal(String printName, Harness harness, @Key(Keys.ufo.banka.banker) Credentials credentials) {
        this.CIF = "3000036007";
        OperationsTasks operationsTasksTasks = harness.tasks().ufo().operations();

        harness.store("PRINT_NAME", printName);
        harness.withUfo()
                .openLoginPage()
                .then(operationsTasksTasks.login().withCredentials(credentials))
                .then(operationsTasksTasks.printTemplateTasks().createPrintTemplateWithCif(CIF, department, category, printName))
                .clickSaveToLocal()
                .clickSendAndClosePrintWindow();
    }

    @AfterEach
    protected void teardown(Harness harness) {
        harness.store(JIRA_COMMENT_STORE_KEY,
                "\nUFO Session ID: " + harness.get("reports.tmfj.execution.commentufosessionid") +
                        "\nNázev dopisu: " + harness.get("PRINT_NAME") +
                        "\nCIF: " + CIF);
    }
}